#!/usr/bin/env python
# # -*- coding: utf-8 -*-
from zotnote.cli import cli
from zotnote.cli import config, add, report, edit, remove
from .config import Configuration


def main():
    # Check if all configuration files are in place
    Configuration.validate()

    # Add commands to CLI
    cli.add_command(add)
    cli.add_command(config)
    cli.add_command(edit)
    cli.add_command(remove)
    # cli.add_command(report)

    # Launch CLI
    cli(prog_name='zotnote')


if __name__ == "__main__":
    main()



#!/usr/bin/env python
fieldIDs = {
    'url': 1,
    'rights': 2,
    'series': 3,
    'volume': 4,
    'issue': 5,
    'edition': 6,
    'place': 7,
    'publisher': 8,
    'pages': 10,
    'ISBN': 11,
    'publicationTitle': 12,
    'ISSN': 13,
    'date': 14,
    'section': 15,
    'callNumber': 18,
    'archiveLocation': 19,
    'distributor': 21,
    'extra': 22,
    'journalAbbreviation': 25,
    'DOI': 26,
    'accessDate': 27,
    'seriesTitle': 28,
    'seriesText': 29,
    'seriesNumber': 30,
    'institution': 31,
    'reportType': 32,
    'code': 36,
    'session': 40,
    'legislativeBody': 41,
    'history': 42,
    'reporter': 43,
    'court': 44,
    'numberOfVolumes': 45,
    'committee': 46,
    'assignee': 48,
    'patentNumber': 50,
    'priorityNumbers': 51,
    'issueDate': 52,
    'references': 53,
    'legalStatus': 54,
    'codeNumber': 55,
    'artworkMedium': 59,
    'number': 60,
    'artworkSize': 61,
    'libraryCatalog': 62,
    'videoRecordingFormat': 63,
    'interviewMedium': 64,
    'letterType': 65,
    'manuscriptType': 66,
    'mapType': 67,
    'scale': 68,
    'thesisType': 69,
    'websiteType': 70,
    'audioRecordingFormat': 71,
    'label': 72,
    'presentationType': 74,
    'meetingName': 75,
    'studio': 76,
    'runningTime': 77,
    'network': 78,
    'postType': 79,
    'audioFileType': 80,
    'versionNumber': 81,
    'system': 82,
    'company': 83,
    'conferenceName': 84,
    'encyclopediaTitle': 85,
    'dictionaryTitle': 86,
    'language': 87,
    'programmingLanguage': 88,
    'university': 89,
    'abstractNote': 90,
    'websiteTitle': 91,
    'reportNumber': 92,
    'billNumber': 93,
    'codeVolume': 94,
    'codePages': 95,
    'dateDecided': 96,
    'reporterVolume': 97,
    'firstPage': 98,
    'documentNumber': 99,
    'dateEnacted': 100,
    'publicLawNumber': 101,
    'country': 102,
    'applicationNumber': 103,
    'forumTitle': 104,
    'episodeNumber': 105,
    'blogTitle': 107,
    'type': 108,
    'medium': 109,
    'title': 110,
    'caseName': 111,
    'nameOfAct': 112,
    'subject': 113,
    'proceedingsTitle': 114,
    'bookTitle': 115,
    'shortTitle': 116,
    'docketNumber': 117,
    'numPages': 118,
    'programTitle': 119,
    'issuingAuthority': 120,
    'filingDate': 121,
    'genre': 122,
    'archive': 123
}



# -*- coding: utf-8 -*-
import shutil
from pathlib import Path

import click
from tomlkit import dumps, loads

from zotnote import config_dir, data_dir, project_dir


class Configuration:
    """A helper class to manage configurations"""
    config_file = config_dir / "config.toml"
    templates_dir = data_dir / "templates"
    pkg_templates_dir = project_dir / "templates"

    @classmethod
    def validate(cls):
        """Validate local configuration & create missing folders and files."""
        if not cls.config_file.exists():
            click.echo("Missing configuration file. Proceeding to create a new one.")
            cls.__create_dir(config_dir)
            cls.create_config()

        if not cls.templates_dir.exists():
            click.echo("App data missing. Proceeding to copy templates.")
            cls.__create_dir(cls.templates_dir)
            cls.__copy_templates()

        pkg_templates = len(list(cls.pkg_templates_dir.glob("*.j2")))
        lc_templates = len(list(cls.templates_dir.glob("*.j2")))

        if lc_templates < pkg_templates:
            click.echo("Template files are missing in app data. Copying...")
            cls.__copy_templates()

    @classmethod
    def load_config(cls):
        """Load configuration file."""

        return loads(cls.config_file.read_text())

    @classmethod
    def create_config(cls):
        """Creates new configuration."""
        if cls.config_file.exists():
            ow = click.confirm("Do you really want to create a new config? "
                               "This will overwrite your existing config.")
            if ow:
                config = cls.__new_config()
        else:
            config = cls.__new_config()
        cls.config_file.write_text(dumps(config))

    @classmethod
    def update_config(cls, key, value):
        config = loads(cls.config_file.read_text())
        config[key] = value
        cls.config_file.write_text(dumps(config))

    @staticmethod
    def __new_config():
        """Load example configuration and populate interactively"""
        exmpl_file = project_dir / "config.toml.example"
        exmpl_config = loads(exmpl_file.read_text())
        config = exmpl_config

        config['name'] = click.prompt("Enter your name")
        config['email'] = click.prompt("Enter your email")

        config["editor"] = click.prompt("The command to execute your editor of choice")

        config["zotero"] = click.prompt("Enter path to your Zotero directory")
        config["notes"] = click.prompt("Enter location for your notes")
        return config

    @staticmethod
    def __create_dir(dir):
        """Create directory if missing."""
        if not dir.exists():
            dir.mkdir(parents=True)

    @classmethod
    def __copy_templates(cls):
        """Copy template files to the data directory."""
        for f in cls.pkg_templates_dir.glob("*.j2"):
            shutil.copyfile(f, cls.templates_dir / f.name)



# -*- coding: utf-8 -*-
from pathlib import Path

from xdg import XDG_CONFIG_HOME, XDG_DATA_HOME

project_dir = Path(__file__).parents[1]
config_dir = Path(XDG_CONFIG_HOME) / "zotnote"
data_dir = Path(XDG_DATA_HOME) / "zotnote"



# -*- coding: utf-8 -*-
import os
import sys
from pathlib import Path

import click
from click_option_group import optgroup, RequiredMutuallyExclusiveOptionGroup

from zotnote.connectors.bbt import (BetterBibtex, BetterBibtexException,
                                    BetterBibtexNotRunning)

from .config import Configuration
from .templates import MarkdownNote
from .utils import citekey_regex


def create_note(citekey, config, bbt, force):
    """Create reading note for CITEKEY in your Zotero library."""
    candidates = bbt.search_citekey_in_bbp(citekey)
    if not candidates:
        click.echo("No results found for " + citekey)
        sys.exit()
    elif len(candidates) != 1:
        click.echo("Something wrong happened here. We have too many candidates...")
        sys.exit()
    else:
        candidate = candidates[0]
        fieldValues = bbt.extract_fields(candidate)

        # Fill template
    md = MarkdownNote(citekey, fieldValues, config)

    # Write output file
    notes_dir = Path(config['notes'])
    outfile = notes_dir / f"{citekey}.md"

    if outfile.exists():
        if force:
            click.echo(f"Overwriting {str(outfile)}")
            outfile.write_text(str(md))
        else:
            choice = click.confirm(
                'This file already exists. Edit instead?" Use --force to overwrite files.')
            if choice:
                os.system(f"{config['editor']} {str(outfile)}")
    else:
        click.echo(f"Writing {str(outfile)}")
        outfile.write_text(str(md))


@click.command()
@click.argument('citekey', required=False)
@click.option("-f", "--force", is_flag=True, help="Overwrite existing notes")
def add(citekey, force):
    """
    Create a new note. If no citekey is provided the Zotero picker is launched.

    CITEKEY is the cite key created by BBT.
    """
    config = Configuration.load_config()

    try:
        bbt = BetterBibtex(config)
    except BetterBibtexNotRunning as e:
        click.echo(e)
        sys.exit()

    if citekey:
        match = citekey_regex.match(citekey)
        if match is None:
            click.echo("The citekey provided is not valid")
            sys.exit()
    else:
        citekey = bbt.citation_picker()
        if citekey is None:
            click.echo("No citation key provided.")
            sys.exit()

    create_note(citekey, config, bbt, force)


@click.command()
@click.argument('citekey', required=False)
def edit(citekey):
    """
    Open a note in your editor of choice.

    CITEKEY is the cite key created by BBT.
    """
    config = Configuration.load_config()

    try:
        bbt = BetterBibtex(config)
    except BetterBibtexNotRunning as e:
        click.echo(e)
        sys.exit()

    if citekey:
        match = citekey_regex.match(citekey)
        if match is None:
            click.echo("The citekey provided is not valid")
            sys.exit()
    else:
        citekey = bbt.citation_picker()
        if citekey is None:
            sys.exit()

    # Write output file
    notes_dir = Path(config['notes'])
    outfile = notes_dir / f"{citekey}.md"

    if outfile.exists():
        os.system(f"{config['editor']} {str(outfile)}")
    else:
        choice = click.confirm("File does not exist yet. Create now?")
        if choice:
            create_note(citekey, config)
        else:
            sys.exit()


@click.command(help="Remove a note")
@click.argument('citekey', required=False)
def remove(citekey):
    """Remove a note.

    CITEKEY is the cite key created by BBT.
    """
    config = Configuration.load_config()

    try:
        bbt = BetterBibtex(config)
    except BetterBibtexNotRunning as e:
        click.echo(e)
        sys.exit()

    if citekey:
        match = citekey_regex.match(citekey)
        if match is None:
            click.echo("The citekey provided is not valid")
            sys.exit()
    else:
        citekey = bbt.citation_picker()
        if citekey is None:
            sys.exit()

    # Write output file
    notes_dir = Path(config['notes'])
    outfile = notes_dir / f"{citekey}.md"

    if outfile.exists():
        choice = click.confirm("Are you sure you want to delete this note?")
        if choice:
            outfile.unlink()
        else:
            sys.exit()
    else:
        click.echo("This note does not exist.")


@click.command()
@optgroup("Edit configuration", cls=RequiredMutuallyExclusiveOptionGroup,
          help="Interact with the configuration")
@optgroup.option("-l", "--list", is_flag=True, help="List all config key/value pairs", )
@optgroup.option("-r", "--reset", is_flag=True, help="Reset config.")
@optgroup.option("-u", "--update-entry", metavar="ENTRY",
                 help="Update an ENTRY in the config file.", type=str)
def config(list, reset, update_entry):
    """Configure Zotnote from the command line.
    """
    config = Configuration.load_config()

    if list:
        for k, v in config.items():
            click.echo(f"{k}: {v}")
    elif reset:
        Configuration.create_config()
    elif update_entry is not None:
        if update_entry in config:
            click.echo(f"Old value: {config[update_entry]}")
            value = click.prompt("New value")
            Configuration.update_config(update_entry, value)
        else:
            click.echo(f"{update_entry} is not a valid entry in the config.")


@click.command()
def report():
    """Create a small, basic report based on the notes.
    """
    NotImplemented


@click.group()
def cli():
    """Automatize and manage your reading notes with Zotero & Better Bibtex Plugin (BBT)
    """



#!/usr/bin/env python
# # -*- coding: utf-8 -*-

import datetime

from jinja2 import Template

from zotnote import data_dir


class MarkdownNote():
    def __init__(self, citekey, fieldValues, config):
        self.citekey = citekey
        self.fieldValues = fieldValues

        self.ts = datetime.datetime.now()
        self.ts_iso = self.ts.isoformat(timespec="seconds")
        self.ts_day = self.ts.strftime("%m.%d.%y")

        self.author = config['name']

        filein = data_dir / "templates/template.j2"
        self.__template = Template(filein.read_text())

    def __str__(self):
        d = {
            'citekey': self.citekey,
            'author': self.author,
            'ts': self.ts_iso,
            'ts_day': self.ts_day,
            'title': self.fieldValues['title'],
            'creator': self.fieldValues['author'],
            'date': self.fieldValues['issued'],
            'doi': self.fieldValues['DOI'],
            'type': self.fieldValues['type']
        }
        return self.__template.render(d)



# # -*- coding: utf-8 -*-
import sqlite3
from pathlib import Path


class ZoteroDBConnector():
    """This class manages the connections to both Zotero's and
    Better Bibtex's local SQLite databases.
    """

    def __init__(self, citekey, config):
        self.__citekey = citekey
        self.__itemID = None

        self.__fieldIDs = {
            'title': 110,
            'DOI': 26,
            'date': 14
        }

        self.__zotero = None

        self.fieldValues = {
            'title': "",
            'type': "",
            'DOI': "",
            'date': "",
            'creator': ""
        }

        self.zotero_dir = Path(config['zotero'])

    def __get_value_id(self):
        results = self.__zotero.execute(
            f"SELECT fieldID, valueID from itemData where itemID=\"{self.itemID}\"")
        return {f: v for (f, v) in results.fetchall()}

    def __get_value_labels(self, valueID):
        result = self.__zotero.execute(
            f"SELECT value from itemDataValues where valueID=\"{valueID}\"")
        return result.fetchone()[0]

    def __get_type_name(self):
        result = self.__zotero.execute(
            f"SELECT itemTypeID from items where itemID=\"{self.itemID}\"")
        itemTypeID = result.fetchone()[0]
        result = self.__zotero.execute(
            f"SELECT typeName from itemTypes where itemTypeID=\"{itemTypeID}\"")
        return result.fetchone()[0]

    def __get_creator_name(self):
        result = self.__zotero.execute(
            f"SELECT creatorID from itemCreators where itemID=\"{self.itemID}\"")
        creators = result.fetchall()
        creatorIDs = [_[0] for _ in creators]

        creators = []
        for creatorID in creatorIDs:
            result = self.__zotero.execute(
                f"SELECT firstName, lastName from creators where creatorID=\"{creatorID}\"")
            name = result.fetchone()
            creators.append(f"{name[1]}, {name[0]}")
        return "; ".join(creators)

    def get_field_values(self):
        self.__zotero = sqlite3.connect(str(self.zotero_dir / "zotero.sqlite.bak"))

        valueIDs = self.__get_value_id()
        for l, fID in self.__fieldIDs.items():
            try:
                self.fieldValues[l] = self.__get_value_labels(valueIDs[fID])
            except Exception:
                pass

        if self.fieldValues['date'] != "":
            self.fieldValues['date'] = self.fieldValues['date'].split(" ")[-1]

        # creator string
        self.fieldValues['creator'] = self.__get_creator_name()

        # item type
        self.fieldValues['type'] = self.__get_type_name()

        # Close the connection
        self.__zotero.close()
        return self.fieldValues



#!/usr/bin/env python
import requests
import json

from zotnote.utils import prune_author_str


class BetterBibtexException(Exception):
    pass


class BetterBibtexNotRunning(Exception):
    pass


class BetterBibtex:
    """
    Wrapper class to access and manage BetterBibtex
    """
    BASE_URL = "http://localhost:23119/better-bibtex/"

    SEARCH_URL = BASE_URL + "json-rpc"
    CAYW_URL = BASE_URL + "cayw"

    def __init__(self, config):
        self.headers = {'Content-Type': "application/json", 'Accept': "application/json"}
        self.payload = [{"jsonrpc": "2.0", "method": "item.search", "params": None}]

        self.selected_fields = ["title", "DOI", "type", "issued", "author"]

        self.author_str_len = 60

        if not self.probe_bbt():
            raise BetterBibtexNotRunning(
                "Better Bibtex is not running. Please make sure to launch Zotero BBT")

    def probe_bbt(self):
        """Check if Zotero & BBT are running
        """
        r = requests.get(BetterBibtex.CAYW_URL + "?probe=probe")
        if r.text == "ready":
            return True
        else:
            return False

    def citation_picker(self):
        """Launch the Zotero citation picker and return result.
        """
        r = requests.get(BetterBibtex.CAYW_URL)
        return r.text

    def search_citekey_in_bbp(self, citekey):
        """
        Searches the endpoint with a citekey.

        Returns all candidates.
        """
        payload = self.payload
        payload[0]['params'] = [f"{citekey}"]
        payload = json.dumps(payload)

        r = requests.post(BetterBibtex.SEARCH_URL,
                          data=payload,
                          headers=self.headers)
        if r.status_code == 200:
            candidates = r.json()[0]['result']
            return candidates
        else:
            return None

    def extract_fields(self, candidate):
        """
        Pretty simple function that retrieves the article information

        Returns a dict defined by selected fields.
        """
        article = {f: None for f in self.selected_fields}

        for f in self.selected_fields:
            if f in candidate:
                if f == "author":
                    author_str = []
                    for name in candidate[f]:
                        name_str = f"{name['family']}, {name['given']}"
                        author_str.append(name_str)
                    author_str = "; ".join(author_str)
                    if len(author_str) >= self.author_str_len:
                        author_str = prune_author_str(author_str,
                                                      self.author_str_len)
                    article[f] = author_str
                elif f == "issued":
                    year = candidate[f]['date-parts'][0][0]
                    article[f] = year
                else:
                    article[f] = candidate[f]
        return article



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zotnote', 'zotnote.connectors']

package_data = \
{'': ['*']}

install_requires = \
['click-option-group>=0.3.0,<0.4.0',
 'click>=7.1.1,<8.0.0',
 'jinja2>=2.11.1,<3.0.0',
 'python-dotenv>=0.12.0,<0.13.0',
 'requests>=2.23.0,<3.0.0',
 'tomlkit>=0.5.11,<0.6.0',
 'xdg>=4.0.1,<5.0.0']

entry_points = \
{'console_scripts': ['zotnote = zotnote.__main__:main']}

setup_kwargs = {
    'name': 'zotnote',
    'version': '0.2.1',
    'description': 'Streamlining reading notes with Zotero',
    'long_description': "# ZotNote\n\nAutomatize and manage your reading notes with Zotero & Better Bibtex Plugin (BBT). **Note: ZotNote is still in early development and not production ready**\n\n[![PyPI version](https://img.shields.io/pypi/v/zotnote.svg)](https://pypi.python.org/pypi/zotnote/)\n\n![ZotNote demo](assets/demo.gif)\n\n---\n\n*Current features*\n\n- Simple installation via pipx/pip\n- Full command-line interface to create, edit, and remove notes\n- Easy selection of entries with interactive Zotero citation picker\n- Templating for the reading notes ([Jinja2](https://jinja.palletsprojects.com/en/2.11.x/)) \n\n*Planned features*\n\n- Annotation of reading notes and individual quotes using tags/keywords\n  - Retrieval of relevant quotes based on these tags and keywords\n  - Analytics based on these tags and keywords\n- Enrich reading notes with more metadata from Zotero\n- Simple reports about progress of literature review \n- (*dreaming*) Automatically export collection of notes as an annotated bibliography.\n\n*Long-term vision*\n\nA literature review suite that connects to Zotero & BBT. Management of reading notes, reading/writing analytics, and basic qualitative text analysis (export reports as HTML via Jupyter notebooks). Export of reading notes in different formats (e.g., annotated bibliography).\n\nYou can find a roadmap for ZotNote [here](https://github.com/Bubblbu/zotnote/projects/1).\n\n## Installation\n\n### Requirements\n\n- [Python](https://www.python.org/downloads/) 3.6 or higher\n- [Zotero Standalone](https://www.zotero.org/) with [Better Bibtex plugin](https://github.com/retorquere/zotero-better-bibtex)\n\n### Recommended: Install via pipx\n\nThe recommended way to install ZotNote is using [pipx](https://pipxproject.github.io/pipx/). Pipx cleanly install the package in an isolated environment (clean uninstalls!) and automagically exposes the CLI-endpoints globally on your system.\n\n```bash\npipx install zotnote\n\n```\n\n\n### Option 2: Install via pip\n\nHowever, you can also simply use pip. Please be aware of the Python version and environments you are using.\n\n```bash\npip install zotnote\n```\n\n### Option 3: Download from GitHub\n\nDownload the latest release from Github and unzip. Put the folder containing the scripts into your `PATH`. \n\nAlternatively, run\n\n```bash\n[sudo] python3 setup.py install\n```\n\nor\n\n```bash\npython3 setup.py install --user\n```\n\n### Option 4: Git clone (for developers)\n\n```bash\ngit clone git@github.com:Bubblbu/zotnote.git\n```\n\nThe project is being developed with [Poetry](https://python-poetry.org/) as a dependency manager.\n\nMore instructions will follow soon!\n\n## Getting started\n\n```shell\nUsage: zotnote [OPTIONS] COMMAND [ARGS]...\n\n  Automatize and manage your reading notes with Zotero & Better Bibtex\n  Plugin (BBT)\n\nOptions:\n  --help  Show this message and exit.\n\nCommands:\n  add     Create a new note.\n  config  Configure Zotnote from the command line.\n  edit    Open a note in your editor of choice.\n  remove  Remove a note\n```\n\n### Configuration\n\nAfter installation you should be able to simply run `zotnote` and be prompted to a quick command-line configuration.\n\nZotNote currently asks you for:\n\n- A name which is used in all reading notes.\n- An email address\n- Path to your Zotero installation (not required anymore. To be removed soon)\n- A folder to store your reading notes\n\n### Usage\n\n`zotnote add [citekey]` should be all you need to start creating your notes. If you don't provide a citekey ZotNote will launch the Zotero picker for you.\n\n## Authors\n\nWritten by [Asura Enkhbayar](https://twitter.com/bubblbu_) while he was quarantined.\n",
    'author': 'Asura Enkhbayar',
    'author_email': 'asura.enkhbayar@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bubblbu/zotnote',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
import re

citekey_regex = re.compile(r"^[a-z0-9]+_[a-z0-9]+_[a-z0-9]+$")


def prune_author_str(string, maxlen):
    """Shorten author string to a certain character length.
    The last author is still shown while middle authors are
    at one point shortened ...
    """
    strings = string.split("; ")
    string = "; ".join(strings[0:-1])[0:maxlen]
    string = string + "...; " + strings[-1]
    return string
